program Rotate(input,output);
{======================================================}
{ Program Rotate used to rotate coordinate file to     }
{ create flaps at different deflections for Panels.    }
{ Written by: Christopher L. Good                      }
{------------------------------------------------------}
{ Modification History                                 }
{  17Dec96  -clg-  Program started.                    }
{======================================================}

uses
  dos, winapi, graph, crt, strings;

{ ----------------------------------------------------------- }

type
  CoordType = record
    x,
    y  : double;
  end;

{ ----------------------------------------------------------- }

var

  infile,
  outfile     : text;

  raw         : array[1..100] of CoordType;

  h,
  Plot_Scale,
  old_angle,
  New_AOA,
  Old_AOA     : double;

  i,
  numpts      : longint;

  cor_file,
  output_file : string[255];
  outstring   : string[5];
  inchar      : char;

  done        : Boolean;

{ ----------------------------------------------------------- }

function arcsin(value:double):double;
{ returns the arcsin of the input value }
begin
  arcsin := arctan(value/sqrt(1.0-sqr(value)));
end;

function arccos(value:double):double;
{ returns the arccos of the input value }
begin
  arccos := arctan(sqrt(1.0-sqr(value))/value);
end;

{ ----------------------------------------------------------- }

procedure Initialize;
const
  graphdriver  : integer = vga;
  graphmode    : integer = vgahi;
  PathToDriver : string  = '';

var
  i           : integer;
begin
  (*
  video driver setup and screen colors
  color table for screen :
     0    1 ....................... 12    13     14     15
    black   greyscale (dark->light)      white  green  red
  *)
  DirectVideo := False;
  InitGraph(GraphDriver, GraphMode, PathToDriver);
  for i:=0 to 15 do SetPalette(i,i);
  for i:=0 to 12 do SetRGBPalette( i,i*5,i*5,i*5);   { black --> (greyscale) }
  SetRGBPalette(13,$ff,$ff,$ff);                     { white }
  SetRGBPalette(14,$00,$ff,$00);                     { green }
  SetRGBPalette(15,$ff,$00,$00);                     { red   }

end;

{ ----------------------------------------------------------- }

begin { main }

  clrscr;
  writeln('Welcome to Rotate ! - written by Captain Chris Good, USAF');
  writeln('---------------------------------------------------------');
  writeln;
  writeln('Enter the input coordinate filename :');
  readln(cor_file);
  writeln;

  {$I-}
  assign(infile,cor_file);
  reset(infile);
  {$I+}
  if (IOResult<>0) or (cor_file='') then
  begin
    writeln('The input coordinate file ''',cor_file,''' does not exist.');
    exit;
  end;

  { read the raw coords in 1..n order }
  i:=1;
  if eof(infile) then exit;
  while not eof(infile) do
  begin
    readln(infile,raw[i].x,raw[i].y);
    i:=i+1;
  end;
  numpts:=i-1;
  close(infile);

  writeln('Enter the output coordinate filename :');
  readln(cor_file);
  writeln;

  assign(outfile,cor_file);
  rewrite(outfile);
  writeln(outfile,'Coordinates (raw) :');
  for i:= 1 to numpts do
    writeln(outfile,i:3,' ',raw[i].x:10:4,'  ',raw[i].y:10:4);
  writeln(outfile);

  Plot_Scale := 500.0;
  Old_AOA  := 0.0;
  New_AOA  := 0.0;
  done := False;
  while (not done) do
  begin

    for i:= 1 to numpts do
    begin
      h := sqrt( sqr(raw[i].x) + sqr(raw[i].y) );
      if (h <> 0.0) then
      begin
        old_angle := 0.0;
        if (raw[i].x = 0.0) then
        begin
          if (raw[i].y > 0.0) then old_angle := 90.0 * PI/180.0;
          if (raw[i].y < 0.0) then old_angle := -90.0 * PI/180.0;
        end
        else
        begin
          if (raw[i].y > 0.0) then old_angle := arccos(raw[i].x/h);
          if (raw[i].y < 0.0) then old_angle := -arccos(raw[i].x/h);
        end;
        raw[i].x  := h * cos(old_angle-Old_AOA+New_AOA);
        raw[i].y  := h * sin(old_angle-Old_AOA+New_AOA);
      end;
    end;

    { opens graphics and sets colors }
    initialize;
    SetColor(14);
    { output airfoil name and header info }
    SetTextStyle(TriplexFont,HorizDir,3);
    Str((-New_AOA*180.0/PI):6:2,outstring);
    OutTextXY(15,100,'Airfoil x/y coords : AOA = '+outstring+' degrees');
    { Set Text Style for all other text }
    SetColor(13);
    SetTextStyle(SmallFont,HorizDir,0);
    SetUserCharSize(8,8,8,8);

    { plot airfoil points }
    for i:=1 to numpts do
    begin
      Str(i,outstring);
      PutPixel (round(raw[i].x*Plot_Scale)+10,300-round(raw[i].y*Plot_Scale),13);
      OutTextXY(round(raw[i].x*Plot_Scale)+10,300-round(raw[i].y*Plot_Scale),outstring);
    end;

    OutTextXY(10,5, 'Press the ''-'' key to decrease the angle of attack.');
    OutTextXY(10,15,'Press the ''+'' key to increase the angle of attack.');
    OutTextXY(10,25,'Press the ''w'' key to write the current coordinates.');
    OutTextXY(10,40,'Press the ''q'' key to quit the program.');

    Old_AOA := New_AOA;
    { wait for user to press 'd','q' to leave graphics screen, or '-'/'+' }
    repeat
      inchar:=Readkey;
      if inchar='w' then                          { write coordinates }
      begin
        writeln(outfile,'Coordinates (AOA = ',(-New_AOA*180.0/PI):6:2,' degrees):');
        for i:= 1 to numpts do
          writeln(outfile,raw[i].x:10:4,'  ',raw[i].y:10:4);
        writeln(outfile);
      end;
      if inchar='q' then done:=True;              { quit program }
      if (inchar='-') or (inchar='_') then        { rotate clockwise }
        New_AOA := Old_AOA + (0.5 * PI/180.0);
      if (inchar='=') or (inchar='+') then        { rotate counter-clockwise }
        New_AOA := Old_AOA - (0.5 * PI/180.0);
    until (inchar='q') or (inchar='d') or
          (inchar='-') or (inchar='_') or
          (inchar='+') or (inchar='=');

  end; { orientation is correct }

  closegraph;
  close(outfile);

end.
